      PROGRAM ROOTF
*
*  This is a polynomial root finder program.
*  the polynomials must have real coefficients.
*  The method applied is a modification of bairstow's 
*  i.e. division by a quadratic polynomial x**2 +p1*x +p2
*  determining p1 & p2 to be such that the remainder is vanishing.
*  Then a deflation is applied by synthetic division to reduce
*  the polynomial degree by two, and the procedure is repeated.
*  A check is always made if the degree is reduced to either
*  one or two to use special finishing procedure.
*
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /TOROOT/ A(0:100), B(0:100), KCO
      COMPLEX ROOT(100), DISC
      PARAMETER ( N = 2, M = 2 )
      DIMENSION ICODE(4), XP(2), XLL(2), XRL(2), IXAT(2)
      CHARACTER LINE*78
      DATA ICODE / 1, 0, 0, 0 /
*
      DO 100 I=1,78
         LINE(I:I) = '='
100   CONTINUE
*
*  Read in, tolerance, polynomial degree, polynomial coefficients
      WRITE (*,*) 'Enter a termination tolerance: '
      READ (*,*) EPS
      WRITE (*,*) 'Enter the degree of the polynomial:'
      READ (*,*) KCO
      WRITE (*,*) 'Enter the polynomial coefficients:( a(0),a(1),...)'
      READ (*,*) (A(I),I=0,KCO)
*
*  Type the above input for reference.
      WRITE (*,'(A)') LINE
      WRITE (*,*) '    Running with the following input  '
      WRITE (*,*) 'Tolerance: ', eps
      WRITE (*,*) 'Degree:    ', kco
      WRITE (*,*)'        Coefficients  a(i), i=0,1,... '
      WRITE (*,*) (A(I),I=0,KCO)
      WRITE (*,'(A)') LINE
      NOR = 0
30    CONTINUE
*  Check if the degree is one, and if so solve a linear equation.
      IF (KCO.EQ.1) THEN
         WRITE (*,*) ' The roots of the polynomial are:'
         NOR = NOR + 1
         ROOT(NOR) = CMPLX(-A(0)/A(1),0)
         DO 1 I=1,NOR
            WRITE (*,19) I, ROOT(I)
1        CONTINUE
19       FORMAT (2X,'Root # ',i4,t17,'(',g14.7,',  ',g14.7,')')
         STOP
      END IF
* Check if the degree is two, and if so solve a quadratic equation.
      IF (KCO.EQ.2) THEN
        WRITE (*,*) ' The roots of the polynomial are:'
        NOR = NOR + 1
        DISC = CMPLX(A(1)**2 - 4*A(2)*A(0),0.D0)
        ROOT(NOR) = (-A(1)+CSQRT(DISC))/2./A(2)
        NOR = NOR + 1
        ROOT(NOR) = (-A(1)-CSQRT(DISC))/2./A(2)
        DO 2 I=1,NOR
           WRITE (*,19) I, ROOT(I)
2       CONTINUE
        STOP
      END IF
*
* Initialize randomly in (-1,1)
      P1 = 2*RANM()-1
      P2 = 2*RANM()-1
      XP(1) = P1
      XP(2) = P2
*
*  Minimize the remainder down to zero !!!
      CALL OPTIMA(N,M,XP,XV,XLL,XRL,IXAT,ICODE,
     &            'in.dat','/dev/null',GRMS,NF,NG,NHE,NJA)
      IF (SQRT(XV).LE.EPS) THEN
        P1 = XP(1)
        P2 = XP(2)
        DISC = CMPLX( P1**2-4*P2, 0.D0)
        NOR = NOR + 1
        ROOT(NOR) = (-P1 + CSQRT(DISC))/2
        NOR = NOR + 1
        ROOT(NOR) = (-P1 - CSQRT(DISC))/2
        CALL SYNDIV(A,KCO,P1,P2,B,R,Q)
        KCO = KCO - 2
        DO 3 I = 0,KCO
           A(I) = B(I)
3       CONTINUE
      END IF
      GO TO 30
      END
*  ---------------------------------------------------------------------
      SUBROUTINE SYNDIV ( A, N, P1, P2, B, R, Q )
*  ---------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
*
*  Given a polynomial: a(n)*x**n + ...+a(1)*x +a(0)
*  and a quadratic: x**2 + p1*x + p2
*  calculate the coefficients of the quotient polynomial: b(k),k=0,1,...,n-2
*  and the remainder terms r and q.
*
      DIMENSION A(0:*), B(0:*)
*
      B(N-2) = A(N)
      B(N-3) = A(N-1)-P1*B(N-2)
      DO 1 K=N-2,2,-1
         B(K-2) = A(K)-P1*B(K-1)-P2*B(K)
1     CONTINUe
      R = A(1) - P1*B(0)-P2*B(1)
      Q = A(0) - P2*B(0)
      END
*  ---------------------------------------------------------------------
      SUBROUTINE SYNDER (A, N, P1, P2, B, R, Q, B1, B2, R1, R2, Q1, Q2)
*  ---------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
*
*  Given a polynomial: a(n)*x**n + ...+a(1)*x +a(0)
*  and a quadratic: x**2 + p1*x + p2
*  calculate the coefficients of the quotient polynomial: b(k),k=0,1,...,n-2
*  and the remainder terms r and q.
*  Calculate the derivatives: b1(k),r1,q1 of b(k),r and q with respect to p1, 
*  and the derivatives: b2(k),r2,q2 of b(k),r and q with respect to p2.
*
      DIMENSION A(0:*),B(0:*),B1(0:*),B2(0:*)
*
      B(N-2) = A(N)
      B(N-3) = A(N-1) - P1*B(N-2)
      DO 1 K=N-2,2,-1
         B(K-2) = A(K) - P1*B(K-1) - P2*B(K)
1     CONTINUE
      R = A(1) - P1*B(0)-P2*B(1)
      Q = A(0) - P2*B(0)
*
      B1(N-2) = 0
      B1(N-3) = -A(N)
      DO 2 K=N-2,2,-1
         B1(K-2) = -B(K-1) - P1*B1(K-1) - P2*B1(K)
2     CONTINUE
      R1 = -B(0) - P1*B1(0) - P2*B1(1)
      Q1 = -P2*B1(0)
*
      B2(N-2) = 0
      B2(N-3) = 0
      DO 3 K=N-2,2,-1
         B2(K-2) = -P1*B2(K-1) - B(K) - P2*B2(K)
3     CONTINUE
      R2 = -P1*B2(0) - B(1) - P2*B2(1)
      Q2 = -B(0) - P2*B2(0)
      END
*  ---------------------------------------------------------------------
      SUBROUTINE SUBSUM ( M, N, X, F )
*  ---------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION X(N), F(M)
      COMMON /TOROOT/ A(0:100), B(0:100), KCO
      P1 = X(1)
      P2 = X(2)
      CALL SYNDIV(A,KCO,P1,P2,B,R,Q)
      F(1) = R
      F(2) = Q
      END
*  ---------------------------------------------------------------------
      SUBROUTINE GRANAL ( N, X, GRAD )
*  ---------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION X(N), GRAD(N)
      COMMON /TOROOT/ A(0:100), B(0:100), KCO
      DIMENSION B1(0:100), B2(0:100)
      P1 = X(1)
      P2 = X(2)
      CALL  SYNDER(A,KCO,P1,P2,B,R,Q,B1,B2,R1,R2,Q1,Q2)
      GRAD(1) = 2*(R*R1 + Q*Q1)
      GRAD(2) = 2*(R*R2 + Q*Q2)
      END
*  ---------------------------------------------------------------------
      SUBROUTINE JANAL ( M, N, X, FJ, LD )
*  ---------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION X(N), FJ(LD,N)
      COMMON /TOROOT/ A(0:100), B(0:100), KCO
      DIMENSION B1(0:100), B2(0:100)
      P1 = X(1)
      P2 = X(2)
      CALL  SYNDER(A,KCO,P1,P2,B,R,Q,B1,B2,R1,R2,Q1,Q2)
      FJ(1,1) = R1
      FJ(1,2) = R2
      FJ(2,1) = Q1
      FJ(2,2) = Q2
      END
